const path = require('path');
const projectRoot = process.cwd();
const PackageMap = require('./PackageMap');
const packagesRoot = path.resolve(projectRoot, './packages');
var fs = require('fs');
const chalk = require('chalk');

const getHump = (word) => {
  return word.replace(/(^\w)|(\-\w)/g, (value) => {
    if(value.indexOf('-') >= 0) {
        return value.replace('-', '').toUpperCase();
    }
    return value.toUpperCase();
  });
}

const propsConfigs = {};
const list = [];
const errors = {
  'errorPackageJsonFile': [], 
  'errorPropsConfigFile': [], 
  'noPropsConfigName': [],
  'error': [],
};
const success = [];
const exposesComponents = {};
let appName = '';

new PackageMap({
  root: packagesRoot,
  level: 1,
  setData: (dirPath, dirName, isFinish) => {
    let package = {}, propsConfig = {};
    try{
      package = require(path.resolve(dirPath, './package.json'));
    }
    catch(e) {
      errors.errorPackageJsonFile.push(dirName);
      return;
    }

    try{
      let content = fs.readFileSync(path.resolve(dirPath, './src/props.config.js'), 'utf-8');
      content = content.replace(/^\s*export\s+default\s*/, '').replace(/\;\s*$/, '');
      propsConfig = (new Function(`return ${content}`))();
    }
    catch(e) {
      errors.errorPropsConfigFile.push(dirName);
      return;
    }

    if(package.name && propsConfig.name) {
      const names = (package.name || '').split('/');

      if(dirName !== names[1]) {
        errors.error.push(`error: The dir name(${dirName}) don't equal to the name of package.json.`);
        return;
      }
      if(!(names[0] && names[1])) {
        errors.error.push(`error: The name don't conform to the format in package.json. 
        For example：@app-name/component-name`);
        return;
      }
      const componentName = getHump(names[1]);
      if(!appName) {
        appName = getHump(names[0].replace(/^\@/, ''));
      }
      
      list.push({
        "component": componentName,
        "name": propsConfig.name,
        "type": "element",
        "appName": appName,
        "packageName": package.name, 
      })

      if(propsConfigs[componentName]) {
        errors.error.push(`error: The name repeat in package.json`);
        return;
      }
      propsConfigs[componentName] = propsConfig;

      exposesComponents[`./${componentName}`] = path.resolve(packagesRoot, names[1], './src')

      success.push({
        dirName: dirName,
        component: componentName,
        name: propsConfig.name
      });
    }
    else {
      errors.noPropsConfigName.push(dirName);
    }
  }
})


const res = {
  propsConfigs,
  list,
  errors,
  success,
  exposesComponents,
  appName
}

if(errors.error && errors.error.length) {
  errors.error.forEach(errorContent => {
    console.log(chalk.red(errorContent));
  })
}

module.exports = res;