import { PureComponent, createElement } from 'react';
import { createBrowserHistory as createHistory } from 'history';
import dva, { connect } from 'dva';

const myApp = dva({
  history: createHistory(),
});

export const getAsyncComponent = (load, options = {}) => {
  return class AsyncComponent extends PureComponent {
    constructor(props) {
      super(props);
      this.state = {};
    }

    loadModel = () => {
      return new Promise((resolve, reject) => {
        const { config = {} } = this.props;
        let loadModel = config.model || options.model;
        if (loadModel && typeof loadModel === 'function') {
          loadModel().then(
            ({ default: model }) => {
              myApp.model(model);
              this.model = model;
              resolve(model.namespace);
            },
            () => {
              resolve(null);
            },
          );
        } else {
          resolve(null);
        }
      });
    };
    componentWillUnmount() {
      if (this.model) {
        myApp.unmodel(this.model);
      }
    }
    componentDidMount() {
      this.loadModel().then(modelName => {
        load().then(({ default: component }) => {
          let newComponent = component;
          if (modelName) {
            newComponent = connect(model => {
              //console.log(9999, model);
              return {
                model: model[modelName],
              };
            })(component);
          }
          this.setState({ component: newComponent });
        });
      });
    }
    render() {
      const { component } = this.state || {};
      return component ? createElement(component) : null;
    }
  };
};

export const getAsyncModel = load => {
  return load;
};

export const app = myApp;
